@@ -35,6 +35,7 @@ use MIME::WordDecoder;
 use MIME::Parser;
 use MIME::Base64;
 use Term::ProgressBar;
+use URI::Escape;
 use constant MAX => 100_000;
 
 use Datasource;
@@ -91,18 +92,18 @@ sub next {
     my $limit_sybase='';
 	## Only the first record found is locked, thanks to the "LIMIT 1" clause
     $order = 'ORDER BY priority_message_bulkmailer ASC, priority_packet_bulkmailer ASC, reception_date_bulkmailer ASC, verp_bulkmailer ASC';
-    if (lc($Conf::Conf{'db_type'}) eq 'mysql' || lc($Conf::Conf{'db_type'}) eq 'pg' || lc($Conf::Conf{'db_type'}) eq 'sqlite'){
+    if (lc($Conf::Conf{'db_type'}) eq 'mysql' || lc($Conf::Conf{'db_type'}) eq 'Pg' || lc($Conf::Conf{'db_type'}) eq 'SQLite'){
 	$order.=' LIMIT 1';
-    }elsif (lc($Conf::Conf{'db_type'}) eq 'oracle'){
+    }elsif (lc($Conf::Conf{'db_type'}) eq 'Oracle'){
 	$limit_oracle = 'AND rownum<=1';
-    }elsif (lc($Conf::Conf{'db_type'}) eq 'sybase'){
+    }elsif (lc($Conf::Conf{'db_type'}) eq 'Sybase'){
 	$limit_sybase = 'TOP 1';
     }
 
     my $statement;
     # Select the most prioritary packet to lock.
     $statement = sprintf "SELECT %s messagekey_bulkmailer AS messagekey, packetid_bulkmailer AS packetid FROM bulkmailer_table WHERE lock_bulkmailer IS NULL AND delivery_date_bulkmailer <= %d %s %s", $limit_sybase, time(), $limit_oracle, $order;
-
+    
     unless ($sth = $dbh->prepare($statement)) {
 	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
 	return undef;
@@ -117,17 +118,28 @@ sub next {
 	return undef;
     }
     $sth->finish();
-
+    
     # Lock the packet previously selected.
-    $statement = sprintf "UPDATE bulkmailer_table SET lock_bulkmailer=%s WHERE messagekey_bulkmailer='%s' AND packetid_bulkmailer='%s'",$dbh->quote($lock), $packet->{'messagekey'}, $packet->{'packetid'};
+    $statement = sprintf "UPDATE bulkmailer_table SET lock_bulkmailer=%s WHERE messagekey_bulkmailer='%s' AND packetid_bulkmailer='%s' AND lock_bulkmailer IS NULL", $dbh->quote($lock), $packet->{'messagekey'}, $packet->{'packetid'};
 
-    unless ($dbh->do($statement)) {
-	do_log('err','Unable to lock bulk packet  SQL statement "%s"; error : %s', $statement, $dbh->errstr);
+    unless ($sth = $dbh->prepare($statement)) {
+	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
+	return undef;
+    }
+    
+    my $rv = $sth->execute;
+    if ($rv < 0) {
+	do_log('err','Unable to lock bulk packet: "%s" : %s', $statement, $dbh->errstr);
+	return undef;
+    }
+    $sth->finish;
+    unless ($rv) {
+	do_log('info','Bulk packet is already locked');
 	return undef;
     }
 
     # select the packet that has been locked previously
-    $statement = sprintf "SELECT messagekey_bulkmailer AS messagekey, packetid_bulkmailer AS packetid, receipients_bulkmailer AS receipients, returnpath_bulkmailer AS returnpath, listname_bulkmailer AS listname, robot_bulkmailer AS robot, priority_message_bulkmailer AS priority_message, priority_packet_bulkmailer AS priority_packet, verp_bulkmailer AS verp, reception_date_bulkmailer AS reception_date, delivery_date_bulkmailer AS delivery_date FROM bulkmailer_table WHERE lock_bulkmailer=%s %s",$dbh->quote($lock), $order;
+    $statement = sprintf "SELECT messagekey_bulkmailer AS messagekey, messageid_bulkmailer AS messageid, packetid_bulkmailer AS packetid, receipients_bulkmailer AS receipients, returnpath_bulkmailer AS returnpath, listname_bulkmailer AS listname, robot_bulkmailer AS robot, priority_message_bulkmailer AS priority_message, priority_packet_bulkmailer AS priority_packet, verp_bulkmailer AS verp, merge_bulkmailer as merge, reception_date_bulkmailer AS reception_date, delivery_date_bulkmailer AS delivery_date FROM bulkmailer_table WHERE lock_bulkmailer=%s %s",$dbh->quote($lock), $order;
 
     unless ($sth = $dbh->prepare($statement)) {
 	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
@@ -138,7 +150,7 @@ sub next {
 	do_log('err','Unable to execute SQL statement "%s" : %s', $statement, $dbh->errstr);
 	return undef;
     }
-
+    
     my $result = $sth->fetchrow_hashref('NAME_lc');
     $sth->finish();
     return $result;
@@ -182,13 +194,209 @@ sub messageasstring {
     my $messageasstring = $sth->fetchrow_hashref('NAME_lc') ;
     $sth->finish;
 
-    return( MIME::Base64::decode($messageasstring->{'message'}) );
+    unless ($messageasstring ){
+	do_log('err',"could not fetch message $messagekey from spool"); 
+    }
+    my $msg = MIME::Base64::decode($messageasstring->{'message'});
+    unless ($msg){
+	do_log('err',"could not decode message $messagekey extrated from spool (base64)"); 
+    }
+    return( $msg );
 }
+#################################"
+# fetch message from bulkspool_table by key 
+#
+sub message_from_spool {
+    my $messagekey = shift;
+    &do_log('debug', '(messagekey : %s)',$messagekey);
+    
+    my $statement = sprintf "SELECT message_bulkspool AS message, messageid_bulkspool AS messageid, dkim_d_bulkspool AS  dkim_d,  dkim_i_bulkspool AS  dkim_i, dkim_privatekey_bulkspool AS dkim_privatekey, dkim_selector_bulkspool AS dkim_selector,dkim_header_list_bulkspool AS dkim_header_list FROM bulkspool_table WHERE messagekey_bulkspool = %s",$dbh->quote($messagekey);
+
+    unless ($dbh and $dbh->ping) {
+	return undef unless &List::db_connect();
+    }
+    unless ($sth = $dbh->prepare($statement)) {
+	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
+	return undef;
+    }
+    
+    unless ($sth->execute) {
+	do_log('err','Unable to execute SQL statement "%s" : %s', $statement, $dbh->errstr);
+	return undef;
+    }
+    my $message_from_spool = $sth->fetchrow_hashref('NAME_lc') ;
+    $sth->finish;
+
+    return({'messageasstring'=> MIME::Base64::decode($message_from_spool->{'message'}),
+	    'messageid' => $message_from_spool->{'messageid'},
+	    'dkim_d' => $message_from_spool->{'dkim_d'},
+	    'dkim_i' => $message_from_spool->{'dkim_i'},
+	    'dkim_selector' => $message_from_spool->{'dkim_selector'},
+	    'dkim_privatekey' => $message_from_spool->{'dkim_privatekey'},
+	    'dkim_header_list' => $message_from_spool->{'dkim_header_list'}});
+
+}
+
+############################################################
+#  merge_msg                                               #
+############################################################
+#  Merge a message with custom attributes of a user.       #
+#                                                          #
+#                                                          #
+#  IN : - MIME:Entity                                      #
+#       - $rcpt : a receipient                             #
+#       - $bulk : HASH                                     #
+#       - $data : HASH with user's data                    #
+#  OUT : 1 | undef                                         #
+#                                                          #
+############################################################
+sub merge_msg {
+
+    my $entity = shift;
+    my $rcpt = shift;
+    my $bulk = shift;
+    my $data = shift;
+
+    ## Test MIME::Entity
+    unless (defined $entity && ref($entity) eq 'MIME::Entity') {
+	&do_log('err', 'echec entity');
+	return undef;
+    }
+
+    my $body;
+    if(defined $entity->bodyhandle){
+	$body      = $entity->bodyhandle->as_string;
+    }
+    ## Get the Content-Type / Charset / Content-Transfer-encoding of a message
+    my $type      = $entity->mime_type;
+    my $charset   = unmime $entity->head->mime_attr('content-type.charset');
+    my $encoding  = unmime $entity->head->mime_encoding;
+
+    my $message_output;
+    my $IO;
+    
+    ## If Content-Type is a text/*
+    if($entity->mime_type =~ /^text/){
+	
+	if(defined $body){
+	    ## --------- Initial Charset to UTF-8 --------- ##
+	    unless($charset =~ /UTF-8/){
+		# Put the charset to UTF-8
+		Encode::from_to($body, $charset, 'UTF-8');
+	      }
+	    ## PARSAGE ##
+	    
+	    &merge_data('rcpt' => $rcpt,
+			'messageid' => $bulk->{'messageid'},
+			'listname' => $bulk->{'listname'},
+			'robot' => $bulk->{'robot'},
+			'data' => $data,
+			'body' => $body,
+			'message_output' => \$message_output,
+			); 
+	    $body = $message_output;
+
+	    unless($charset =~ /UTF-8/){
+		# Put the charset to the initial
+		Encode::from_to($body, 'UTF-8',$charset);
+	      }
+	    
+	    # Write the new body in the entity
+	    unless($IO = $entity->bodyhandle->open("w") || die "open body: $!"){
+		&do_log('err', "Can't open Entity");
+		return undef;
+	    }
+	    unless($IO->print($body)){
+		&do_log('err', "Can't write in Entity");
+		return undef;
+	    }
+	    unless($IO->close || die "close I/O handle: $!"){
+		&do_log('err', "Can't close Entity");
+		return undef;
+	    }
+	}
+    }
+    
+    ##--- Recursive call of the method. ---##
+    ## Course on the different parts of the message at all levels. 
+    foreach my $part ($entity->parts) {
+	unless(&merge_msg($part, $rcpt, $bulk, $data)){
+	    &do_log('err', "Failed to merge message part.");
+	    return undef;
+	}  
+    }
+
+    return 1;
+
+}
+
+############################################################
+#  merge_data                                              #
+############################################################
+#  This function retrieves the customized data of the      #
+#  users then parse the message. It returns the message    #
+#  personalized to bulk.pl                                 #
+#  It uses the method &tt2::parse_tt2                      #
+#  It uses the method &List::get_subscriber_no_object      #
+#  It uses the method &tools::get_fingerprint              #
+#                                                          #
+# IN : - rcpt : the receipient email                       #
+#      - listname : the name of the list                   #
+#      - robot : the host                                  #
+#      - data : HASH with many data                        #
+#      - body : message with the TT2                       #
+#      - message_output : object, IO::Scalar               #
+#                                                          #
+# OUT : - message_output : customized message              #
+#     | undef                                              #
+#                                                          #
+############################################################ 
+sub merge_data {
+
+    my %params = @_;
+    my $rcpt = $params{'rcpt'},
+    my $listname = $params{'listname'},
+    my $robot = $params{'robot'},
+    my $data = $params{'data'},
+    my $body = $params{'body'},
+    my $message_output = $params{'message_output'},
+    
+    my $options;
+    $options->{'is_not_template'} = 1;
+    
+    my $user_details;
+    $user_details->{'email'} = $rcpt;
+    $user_details->{'name'} = $listname;
+    $user_details->{'domain'} = $robot;
+    
+    # get_subscriber_no_object() return the user's details with the custom attributes
+    my $user = &List::get_subscriber_no_object($user_details);
+
+    $user->{'escaped_email'} = &URI::Escape::uri_escape($rcpt);
+    $user->{'friendly_date'} = gettext_strftime("%d %b %Y  %H:%M", localtime($user->{'date'}));
+
+    # this method as been removed because some users may forward authentication link
+    # $user->{'fingerprint'} = &tools::get_fingerprint($rcpt);
+
+    $data->{'user'} = $user;
+    $data->{'robot'} = $robot;
+    $data->{'listname'} = $listname;
+
+    # Parse the TT2 in the message : replace the tags and the parameters by the corresponding values
+    unless (&tt2::parse_tt2($data,\$body, $message_output, '', $options)) {
+	&do_log('err','Unable to parse body : "%s"', \$body);
+	return undef;
+    }
+
+    return 1;
+}
+
 ## 
 sub store { 
     my %data = @_;
     
     my $msg = $data{'msg'};
+    my $msg_id = $data{'msg_id'};
     my $rcpts = $data{'rcpts'};
     my $from = $data{'from'};
     my $robot = $data{'robot'};
@@ -198,8 +406,12 @@ sub store {
     my $delivery_date = $data{'delivery_date'};
     my $verp  = $data{'verp'};
     $verp=0 unless($verp);
-    
-    &do_log('debug', 'Bulk::store(<msg>,<rcpts>,from = %s,robot = %s,listname= %s,priority_message = %s, priority_packet = %s,delivery_date= %s,verp = %s)',$from,$robot,$listname,$priority_message,$priority_packet,$delivery_date,$verp);
+    my $merge  = $data{'merge'};
+    $merge=0 unless($merge);
+    my $dkim = $data{'dkim'};
+    my $tag_as_last = $data{'tag_as_last'};
+
+    &do_log('debug', 'Bulk::store(<msg>,<rcpts>,from = %s,robot = %s,listname= %s,priority_message = %s, delivery_date= %s,verp = %s, merge = %s, dkim: d= %s i=%s, last: %s)',$from,$robot,$listname,$priority_message,$delivery_date,$verp,$merge,$dkim->{'d'},$dkim->{'i'},$tag_as_last);
 
     $dbh = &List::db_get_handler();
 
@@ -215,12 +427,19 @@ sub store {
 
     my $messagekey = &tools::md5_fingerprint($msg);
 
-    # first store the message in bulk_message_table (because as soon as packet are created bulk.pl may distribute them).
-    # if messagekey is equal to message_fingerprint, the message is already stored in database
+    # first store the message in bulk_spool_table 
+    # because as soon as packet are created bulk.pl may distribute them
+    # Compare the current message finger print to the fingerprint
+    # of the last call to store() ($message_fingerprint is a global var)
+    # If fingerprint is the same, then the message should not be stored
+    # again in bulkspool_table
     
     my $message_already_on_spool;
 
-    if ($messagekey ne $message_fingerprint) {
+    if ($messagekey eq $message_fingerprint) {
+	$message_already_on_spool = 1;
+	
+    }else {
 
 	## search if this message is already in spool database : mailfile may perform multiple submission of exactly the same message 
 	my $statement = sprintf "SELECT count(*) FROM bulkspool_table WHERE ( messagekey_bulkspool = %s )", $dbh->quote($messagekey);
@@ -238,13 +457,17 @@ sub store {
 	$sth->finish();
 	
 	# if message is not found in bulkspool_table store it
-	if ($message_already_on_spool == 0) {	    
-	    my $statement = sprintf "INSERT INTO bulkspool_table (messagekey_bulkspool, message_bulkspool, lock_bulkspool) VALUES (%s, %s, '1')",$dbh->quote($messagekey),$dbh->quote($msg);
-	    my $statementtrace = sprintf "INSERT INTO bulkspool_table (messagekey_bulkspool, message_bulkspool, lock_bulkspool) VALUES (%s, %s, '1')",$dbh->quote($messagekey),$dbh->quote(substr($msg, 0, 100));	    
-	    unless ($dbh->do($statement)) {
-		do_log('err','Unable to add message in bulkspool_table "%s"; error : %s', $statementtrace, $dbh->errstr);
-		return undef;
-	    }
+	if ($message_already_on_spool == 0) {
+	    my $statement      = sprintf "INSERT INTO bulkspool_table (messagekey_bulkspool, messageid_bulkspool, message_bulkspool, lock_bulkspool, dkim_d_bulkspool,dkim_i_bulkspool,dkim_selector_bulkspool, dkim_privatekey_bulkspool,dkim_header_list_bulkspool) VALUES (%s, %s, %s, 1, %s, %s, %s ,%s ,%s)",$dbh->quote($messagekey),$dbh->quote($msg_id),$dbh->quote($msg),$dbh->quote($dkim->{d}), $dbh->quote($dkim->{i}),$dbh->quote($dkim->{selector}),$dbh->quote($dkim->{private_key}), $dbh->quote($dkim->{header_list}); 
+
+	    my $statementtrace = sprintf "INSERT INTO bulkspool_table (messagekey_bulkspool, messageid_bulkspool, message_bulkspool, lock_bulkspool, dkim_d_bulkspool, dkim_i_bulkspool, dkim_selector_bulkspool, dkim_privatekey_bulkspool, dkim_header_list_bulkspool) VALUES (%s, %s, %s, 1, %s ,%s ,%s, %s, %s)",$dbh->quote($messagekey),$dbh->quote($msg_id),$dbh->quote(substr($msg, 0, 100)), $dbh->quote($dkim->{d}), $dbh->quote($dkim->{i}),$dbh->quote($dkim->{selector}),$dbh->quote(substr($dkim->{private_key},0,30)), $dbh->quote($dkim->{header_list});  
+	    # do_log('debug',"insert : $statement_trace");
+
+		unless ($dbh->do($statement)) {
+		    do_log('err','Unable to add message in bulkspool_table "%s"; error : %s', $statementtrace, $dbh->errstr);
+		    return undef;
+		}
+
 	    $message_fingerprint = $messagekey;
 	}
     }
@@ -262,12 +485,13 @@ sub store {
     }
 
     my $priority_for_packet;
+    my $already_tagged = 0;
     my $packet_rank = 0; # Initialize counter used to check wether we are copying the last packet.
     foreach my $packet (@{$rcpts}) {
-	if($packet_rank == $#{$rcpts} && !$verp){
-	    $priority_for_packet = $priority_packet+1;
-	}else{
-	    $priority_for_packet = $priority_packet;
+	$priority_for_packet = $priority_packet;
+	if($tag_as_last && !$already_tagged){
+	    $priority_for_packet = $priority_packet + 5;
+	    $already_tagged = 1;
 	}
 	$type = ref $packet;
 	my $rcptasstring ;
@@ -292,8 +516,12 @@ sub store {
 	    $packet_already_exist = $sth->fetchrow;
 	    $sth->finish();
 	}
-	unless ($packet_already_exist) {
-	    my $statement = sprintf "INSERT INTO bulkmailer_table (messagekey_bulkmailer,packetid_bulkmailer,receipients_bulkmailer,returnpath_bulkmailer,robot_bulkmailer,listname_bulkmailer, verp_bulkmailer, priority_message_bulkmailer, priority_packet_bulkmailer, reception_date_bulkmailer, delivery_date_bulkmailer) VALUES (%s,%s,%s,%s,%s,%s,'%s','%s','%s','%s','%s')", $dbh->quote($messagekey),$dbh->quote($packetid),$dbh->quote($rcptasstring),$dbh->quote($from),$dbh->quote($robot),$dbh->quote($listname),$verp,$priority_message, $priority_for_packet, $current_date,$delivery_date;
+	 
+	 if ($packet_already_exist) {
+	     do_log('err','Duplicate message not stored in bulmailer_table');
+	     
+	 }else {
+	    my $statement = sprintf "INSERT INTO bulkmailer_table (messagekey_bulkmailer,messageid_bulkmailer,packetid_bulkmailer,receipients_bulkmailer,returnpath_bulkmailer,robot_bulkmailer,listname_bulkmailer, verp_bulkmailer, merge_bulkmailer, priority_message_bulkmailer, priority_packet_bulkmailer, reception_date_bulkmailer, delivery_date_bulkmailer) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)", $dbh->quote($messagekey),$dbh->quote($msg_id),$dbh->quote($packetid),$dbh->quote($rcptasstring),$dbh->quote($from),$dbh->quote($robot),$dbh->quote($listname),$verp,$merge,$priority_message, $priority_for_packet, $current_date,$delivery_date;
 	    unless ($sth = $dbh->do($statement)) {
 		do_log('err','Unable to add packet in bulkmailer_table "%s"; error : %s', $statement, $dbh->errstr);
 		return undef;
@@ -355,7 +583,7 @@ sub remove_bulkspool_message {
 	return undef unless &List::db_connect();
     }
 
-    my $statement = sprintf "DELETE FROM `%s` WHERE `%s` = '%s'",$table,$key,$messagekey;
+    my $statement = sprintf "DELETE FROM %s WHERE %s = '%s'",$table,$key,$messagekey;
 
     unless ($sth = $dbh->prepare($statement)) {
 	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
@@ -385,8 +613,9 @@ sub store_test {
     my $priority_message = 9;
     my $delivery_date = time;
     my $verp  = 1;
+    my $merge  = 1;
     
-    &do_log('debug', 'Bulk::store_test(<msg>,<rcpts>,from = %s,robot = %s,listname= %s,priority_message = %s,delivery_date= %s,verp = %s)',$from,$robot,$listname,$priority_message,$delivery_date,$verp);
+    &do_log('debug', 'Bulk::store_test(<msg>,<rcpts>,from = %s,robot = %s,listname= %s,priority_message = %s,delivery_date= %s,verp = %s, merge = %s)',$from,$robot,$listname,$priority_message,$delivery_date,$verp,$merge);
 
     print "maxtest: $maxtest\n";
     print "barmax: $barmax\n";
