@@ -1,5 +1,5 @@
 # Auth.pm - This module provides web authentication functions
-# RCS Identication ; $Revision: 5207 $ ; $Date: 2008-11-07 16:01:01 +0100 (ven, 07 nov 2008) $ 
+# RCS Identication ; $Revision: 6355 $ ; $Date: 2010-01-26 17:11:03 +0100 (mar 26 jan 2010) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -21,21 +21,27 @@
 
 
 package Auth;
-use lib '--LIBDIR--';
 
-use Exporter;
-our @ISA = ('Exporter');
-our @EXPORT = ();
+use Digest::MD5;
 
+use Language;
 use Log;
 use Conf;
 use List;
 use report;
-use Digest::MD5;
 
-# use Net::SSLeay qw(&get_https);
-# use Net::SSLeay;
+## return the password finger print (this proc allow futur replacement of md5 by sha1 or ....)
+sub password_fingerprint{
 
+    do_log('debug', 'Auth::password_fingerprint');
+
+    my $pwd = shift;
+    if(&Conf::get_robot_conf('*','password_case') eq 'insensitive') {
+	return &tools::md5_fingerprint(lc($pwd));
+    }else{
+	return &tools::md5_fingerprint($pwd);
+    }    
+}
 
 
 ## authentication : via email or uid
@@ -49,18 +55,17 @@ use Digest::MD5;
 
      if( &tools::valid_email($auth)) {
 	 return &authentication($robot, $auth,$pwd);
-
      }else{
 	 ## This is an UID
-       foreach my $ldap (@{$Conf{'auth_services'}{$robot}}){
-	 # only ldap service are to be applied here
-	 next unless ($ldap->{'auth_type'} eq 'ldap');
-	 
-	 $canonic = &ldap_authentication($robot, $ldap, $auth,$pwd,'uid_filter');
-	 last if ($canonic); ## Stop at first match
-       }
-       if ($canonic){
-
+	 foreach my $ldap (@{$Conf{'auth_services'}{$robot}}){
+	     # only ldap service are to be applied here
+	     next unless ($ldap->{'auth_type'} eq 'ldap');
+	     
+	     $canonic = &ldap_authentication($robot, $ldap, $auth,$pwd,'uid_filter');
+	     last if ($canonic); ## Stop at first match
+	 }
+	 if ($canonic){
+	     
 	     unless($user = &List::get_user_db($canonic)){
 		 $user = {'email' => $canonic};
 	     }
@@ -106,26 +111,31 @@ sub authentication {
 
 
     unless ($user = &List::get_user_db($email)) {
-	$user = {'email' => $email,
-		 'password' => &tools::tmp_passwd($email)
-		 };
+	$user = {'email' => $email };
     }    
     unless ($user->{'password'}) {
-	$user->{'password'} = &tools::tmp_passwd($email);
+	$user->{'password'} = '';
     }
     
-
-
+    if ($user->{'wrong_login_count'} > &Conf::get_robot_conf($robot, 'max_wrong_password')){
+	# too many wrong login attemp
+	&List::update_user_db($email,{wrong_login_count => $user->{'wrong_login_count'}+1}) ;
+	&report::reject_report_web('user','too_many_wrong_login',{}) unless ($ENV{'SYMPA_SOAP'});
+	&do_log('err','login is blocked : too many wrong password submission for %s', $email);
+	return undef;
+    }
     foreach my $auth_service (@{$Conf{'auth_services'}{$robot}}){
+	next if ($auth_service->{'auth_type'} eq 'authentication_info_url');
 	next if ($email !~ /$auth_service->{'regexp'}/i);
 	next if (($email =~ /$auth_service->{'negative_regexp'}/i)&&($auth_service->{'negative_regexp'}));
 
 	## Only 'user_table' and 'ldap' backends will need that Sympa collects the user passwords
 	## Other backends are Single Sign-On solutions
 	if ($auth_service->{'auth_type'} eq 'user_table') {
+	    my $fingerprint = &password_fingerprint ($pwd);	    	    
 	    
-	    if(((&Conf::get_robot_conf('*','password_case') eq 'insensitive') && (lc($pwd) eq lc($user->{'password'}))) || 
-	       ($pwd eq $user->{'password'})) {
+	    if ($fingerprint eq $user->{'password'}) {
+		&List::update_user_db($email,{wrong_login_count => 0}) ;
 		return {'user' => $user,
 			'auth' => 'classic',
 			'alt_emails' => {$email => 'classic'}
@@ -136,6 +146,7 @@ sub authentication {
 		unless($user = &List::get_user_db($canonic)){
 		    $user = {'email' => $canonic};
 		}
+		&List::update_user_db($canonic,{wrong_login_count => 0}) ;
 		return {'user' => $user,
 			'auth' => 'ldap',
 			'alt_emails' => {$email => 'ldap'}
@@ -144,21 +155,9 @@ sub authentication {
 	}
     }
 
-    ## If web context and password has never been changed
-    ## Then prompt user
-    unless ($ENV{'SYMPA_SOAP'}) {
-	foreach my $auth_service (@{$Conf{'auth_services'}{$robot}}){
-	    next unless ($email !~ /$auth_service->{'regexp'}/i);
-	    next unless (($email =~ /$auth_service->{'negative_regexp'}/i)&&($auth_service->{'negative_regexp'}));
-	    if ($auth_service->{'auth_type'} eq 'user_table') {
-		if ($user->{'password'} =~ /^init/i) {
-		    &report::reject_report_web('user','init_passwd',{});
-		    last;
-		}
-	    }
-	}
-    }
-    
+    # increment wrong login count.
+    &List::update_user_db($email,{wrong_login_count =>$user->{'wrong_login_count'}+1}) ;
+
     &report::reject_report_web('user','incorrect_passwd',{}) unless ($ENV{'SYMPA_SOAP'});
     &do_log('err','authentication: incorrect password for user %s', $email);
 
@@ -171,7 +170,8 @@ sub authentication {
 sub ldap_authentication {
      my ($robot, $ldap, $auth, $pwd, $whichfilter) = @_;
      my ($mesg, $host,$ldap_passwd,$ldap_anonymous);
-     &do_log('debug2','Auth::ldap_authentication(%s,%s,%s)', $auth,$pwd,$whichfilter);
+     &do_log('debug2','Auth::ldap_authentication(%s,%s,%s)', $auth,'****',$whichfilter);
+     &do_log('debug3','Password used: %s',$pwd);
 
      unless (&tools::get_filename('etc',{},'auth.conf', $robot)) {
 	 return undef;
@@ -197,7 +197,6 @@ sub ldap_authentication {
      $filter =~ s/\[sender\]/$auth/ig;
      
      ## bind in order to have the user's DN
-     my $ldap_anonymous;
      my $param = &tools::dup_var($ldap);
      my $ds = new Datasource('LDAP', $param);
      
@@ -223,15 +222,14 @@ sub ldap_authentication {
      $ds->disconnect();
      
      ##  bind with the DN and the pwd
-     my $ldap_passwd;
      
      ## Duplicate structure first
      ## Then set the bind_dn and password according to the current user
-     my $param = &tools::dup_var($ldap);
+     $param = &tools::dup_var($ldap);
      $param->{'ldap_bind_dn'} = $DN[0];
      $param->{'ldap_bind_password'} = $pwd;
      
-     my $ds = new Datasource('LDAP', $param);
+     $ds = new Datasource('LDAP', $param);
      
      unless (defined $ds && ($ldap_passwd = $ds->connect())) {
        do_log('err',"Unable to connect to the LDAP server '%s'", $param->{'host'});
@@ -282,9 +280,14 @@ sub ldap_authentication {
      
      $ds->disconnect() or &do_log('notice', "unable to unbind");
      &do_log('debug3',"canonic: $canonic_email[0]");
-     return lc($canonic_email[0]);
-     
- }
+     ## If the identifier provided was a valid email, return the provided email.
+     ## Otherwise, return the canonical email guessed after the login.
+     if( &tools::valid_email($auth)) {
+	 return ($auth);
+     }else{
+	 return lc($canonic_email[0]);
+     } 
+}
 
 
 # fetch user email using his cas net_id and the paragrapah number in auth.conf
@@ -387,7 +390,99 @@ sub remote_app_check_password {
     return undef;
 }
  
-  
+# create new entry in one_time_ticket table using a rand as id so later access is authenticated
+#
+
+sub create_one_time_ticket {
+    my $email = shift;
+    my $robot = shift;
+    my $data_string = shift;
+    my $remote_addr = shift; ## Value may be 'mail' if the IP address is not known
+
+    my $ticket = &SympaSession::get_random();
+    do_log('info', 'Auth::create_one_time_ticket(%s,%s,%s,%s) value = %s',$email,$robot,$data_string,$remote_addr,$ticket);
+
+    my $date = time;
+    my $dbh = &List::db_get_handler();
+    my $sth;
+
+    ## Check database connection
+    unless ($dbh and $dbh->ping) {
+	return undef unless &List::db_connect();
+    }
+    
+    my $statement = sprintf "INSERT INTO one_time_ticket_table (ticket_one_time_ticket, robot_one_time_ticket, email_one_time_ticket, date_one_time_ticket, data_one_time_ticket, remote_addr_one_time_ticket, status_one_time_ticket) VALUES ('%s','%s','%s','%s','%s','%s','%s')",$ticket,$robot,$email,time,$data_string,$remote_addr,'open';
+
+    unless ($dbh->do($statement)) {
+	do_log('err','Unable to insert in table one_time_ticket_table while executing SQL statement "%s" : %s', $statement, $dbh->errstr);
+	return undef;
+    }   
+    return $ticket;
+}
+
+# read one_time_ticket from table and remove it
+#
+sub get_one_time_ticket {
+    my $ticket_number = shift;
+    my $addr = shift; 
+    
+    do_log('debug2', '(%s)',$ticket_number);
+    
+    my $dbh = &List::db_get_handler();
+    my $sth;
+    
+    ## Check database connection
+    unless ($dbh and $dbh->ping) {
+	return return {'result'=>'error'} unless &List::db_connect();
+    }
+    my $statement;
+    $statement = sprintf "SELECT ticket_one_time_ticket AS ticket, robot_one_time_ticket AS robot, email_one_time_ticket AS email, date_one_time_ticket AS \"date\", data_one_time_ticket AS data, remote_addr_one_time_ticket AS remote_addr, status_one_time_ticket as status FROM one_time_ticket_table WHERE ticket_one_time_ticket = '%s' ", $ticket_number;
+    
+    unless ($sth = $dbh->prepare($statement)) {
+	do_log('err','Auth::get_one_time_ticket: Unable to prepare SQL statement : %s', $dbh->errstr);
+	return {'result'=>'error'};
+    }
+    unless ($sth->execute) {
+	do_log('err','Auth::get_one_time_ticket: Unable to execute SQL statement "%s" : %s', $statement, $dbh->errstr);
+	return {'result'=>'error'};
+    }    
+ 
+    my $ticket = $sth->fetchrow_hashref('NAME_lc');
+    $sth->finish();
+    
+    unless ($ticket) {	
+	do_log('info','Auth::get_one_time_ticket: Unable to find one time ticket %s (SQL query %s)%s', $ticket,$statement, $dbh->errstr);
+	return {'result'=>'not_found'};
+    }
+    
+    my $result;
+    my $printable_date = gettext_strftime "%d %b %Y at %H:%M:%S", localtime($ticket->{'date'});
+
+    if ($ticket->{'status'} ne 'open') {
+	$result = 'closed';
+	do_log('info','Auth::get_one_time_ticket: ticket %s from %s has been used before (%s)',$ticket_number,$ticket->{'email'},$printable_date);
+    }
+    elsif (time - $ticket->{'date'} > 48 * 60 * 60) {
+	do_log('info','Auth::get_one_time_ticket: ticket %s from %s refused because expired (%s)',$ticket_number,$ticket->{'email'},$printable_date);
+	$result = 'expired';
+    }else{
+	$result = 'success';
+    }
+    $statement = sprintf "UPDATE one_time_ticket_table SET status_one_time_ticket = '%s' WHERE (ticket_one_time_ticket='%s')", $addr, $ticket_number;
+    
+    unless ($dbh->do($statement)) {
+    	do_log('err','Auth::get_one_time_ticket  Unable to execute SQL statement "%s" : %s', $statement, $dbh->errstr);
+    }
+
+    do_log('info', 'xxxx Auth::get_one_time_ticket(%s) : result : %s',$ticket_number,$result);
+    return {'result'=>$result,
+	    'date'=>$ticket->{'date'},
+	    'email'=>$ticket->{'email'},
+	    'remote_addr'=>$ticket->{'remote_addr'},
+	    'robot'=>$ticket->{'robot'},
+	    'data'=>$ticket->{'data'},
+	    'status'=>$ticket->{'status'}
+	};
+}
+    
 1;
-
-
